package net.sf.itcb.addons.monitoring.track.portlet.interceptor;

import javax.portlet.PortletRequest;
import javax.servlet.http.HttpServletRequest;

import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Required;

import net.sf.itcb.addons.monitoring.track.MonitoringTransactionManager;
import net.sf.itcb.addons.monitoring.track.bean.ItcbTrackBean;
import net.sf.itcb.common.business.core.ItcbConstants;
import net.sf.itcb.common.portlet.vaadin.ItcbVaadinApplication;
import net.sf.itcb.common.portlet.vaadin.interceptor.request.ItcbRequestInterceptor;

/**
 * This interceptor's aim is to track each request received by the portlet or the servlet.<br/>
 * It can be usefull in development mode but should be used in Production mode.<br/>
 * In production mode, {@link TrackerNoTrackRequestInterceptor} should be used instead in order to only clean transactionId before leaving the request.
 * @author Pierre Le Roux
 */
public class TrackerRequestInterceptor implements ItcbRequestInterceptor {

	private static final String HANDLE_REQUEST = "handleRequest";
	private static final String NET_SF_ITCB_ADDONS_MONITORING_TRACK_PORTLET_INTERCEPTOR_TRACKER_REQUEST_INTERCEPTOR = "net.sf.itcb.addons.monitoring.track.portlet.interceptor.TrackerRequestInterceptor";

	MonitoringTransactionManager monitoringTransactionManager;
	
	@Required
	public void setMonitoringTransactionManager(MonitoringTransactionManager monitoringTransactionManager) {
		this.monitoringTransactionManager=monitoringTransactionManager;
	}
		
	@Override
	public void handleRequestStart(ItcbVaadinApplication application,
			HttpServletRequest servletRequest) {
		long startTime = System.currentTimeMillis();
		  
		ItcbTrackBean itcbTrackBean = new ItcbTrackBean();
		itcbTrackBean.setClassName(NET_SF_ITCB_ADDONS_MONITORING_TRACK_PORTLET_INTERCEPTOR_TRACKER_REQUEST_INTERCEPTOR);

		itcbTrackBean.setStartTime(startTime);

		
		monitoringTransactionManager.setCurrentTrackBean(itcbTrackBean);

	}

	@Override
	public void handleRequestStart(ItcbVaadinApplication application,
			PortletRequest portletRequest) {
		long startTime = System.currentTimeMillis();
		  
		ItcbTrackBean itcbTrackBean = new ItcbTrackBean();
		itcbTrackBean.setClassName(NET_SF_ITCB_ADDONS_MONITORING_TRACK_PORTLET_INTERCEPTOR_TRACKER_REQUEST_INTERCEPTOR);

		itcbTrackBean.setStartTime(startTime);


		
		monitoringTransactionManager.setCurrentTrackBean(itcbTrackBean);

	}

	@Override
	public void handleRequestEnd(ItcbVaadinApplication application,
			HttpServletRequest servletRequest) {
		ItcbTrackBean itcbTrackBean = (ItcbTrackBean)servletRequest.getAttribute(NET_SF_ITCB_ADDONS_MONITORING_TRACK_PORTLET_INTERCEPTOR_TRACKER_REQUEST_INTERCEPTOR);
	    itcbTrackBean.setTimeInMillis(System.currentTimeMillis() - itcbTrackBean.getStartTime());
	    itcbTrackBean.setMethod(HANDLE_REQUEST);
	    itcbTrackBean.setUser(MDC.get(ItcbConstants.MDC_USER_KEY));
	    monitoringTransactionManager.endCurrentTrackBean();
	}

	@Override
	public void handleRequestEnd(ItcbVaadinApplication application,
			PortletRequest portletRequest) {
	ItcbTrackBean itcbTrackBean = (ItcbTrackBean)portletRequest.getAttribute(NET_SF_ITCB_ADDONS_MONITORING_TRACK_PORTLET_INTERCEPTOR_TRACKER_REQUEST_INTERCEPTOR);
		
	    itcbTrackBean.setTimeInMillis(System.currentTimeMillis() - itcbTrackBean.getStartTime());
	    itcbTrackBean.setMethod(HANDLE_REQUEST);
	    itcbTrackBean.setUser(MDC.get(ItcbConstants.MDC_USER_KEY));
	    monitoringTransactionManager.endCurrentTrackBean();
	}

}
